from __future__ import print_function
import click
import glob
import os
from os.path import join

from mcutk.debugger import getdebugger
from mcutk.managers.conf_mgr import ConfMgr
from mcutk.managers.debug_helper import Debughelper


@click.command('gdbserver', short_help='start a standalone gdbserver')
@click.option('-t', '--type', default='jlink', help='debugger type, jlink/pyocd.')
@click.option('-u', '--usbid', help='unique usb id')
@click.option('-p', '--port', help='specific server port')
def cli(type, usbid, port):
    """ Start a gdb server."""

    cfger = ConfMgr.load()
    debugger, device = Debughelper.choose_device(None)
    if port:
        device['port'] = port

    if debugger and device:
        debugger.start_gdbserver(**device)




import os
import sys
import time
import logging
import serial
from xmodem import XMODEM

from mcutk.exceptions import SerialCannotOpenException


def loadfile(serialo, filename, ctrltext='x'):
    """Flash programming with serial based on xmodem protocl.

    Arguments:
        serialo {serial.Serial object} -- an serial.Serial object
        filename {str} -- path to file.
        ctrltext {str} -- the string will be written to serial to control the loader mode.
                        default is 'x'

    Exceptions:
        SerialCannotOpenException -- raise when serial cannot be open

    Returns:
        boolean -- There is no error, it will return True, else False.
    """

    if not serialo.is_open:
        serialo.open()

    cnt, dat = 0, ""

    logging.info("checking serial output")
    # read data when 'READY' is in the context, or timeout.
    while True:
        n = serialo.inWaiting()
        dat = serialo.read(n)

        # logging the serial ouput
        if dat != "":
            logging.info("\n" +dat)

        # check string
        if "READY" in dat:
            break

        # check timeout
        if cnt > 60:
            logging.warning("Timeout: not found 'READY' in serial output..")
            serialo.close()
            return False

        cnt = cnt + 1
        time.sleep(1)


    def getc(size, timeout=10):
        return serialo.read(size)


    def putc(data, timeout=10):
        sys.stdout.write('=')
        serialo.write(data, log=False)
        time.sleep(0.001)

    serialo.write(ctrltext)
    logging.info("> Download program via XMODEM protocol: %s", filename)

    result = False
    with open(filename, 'rb') as stream:
        modem = XMODEM(getc, putc)
        result = modem.send(stream, retry=4)
        serialo.close()

    if result:
        logging.info("Flash successful")
    else:
        logging.warning("Flash failure!!!")

    return result

def loadfile_delayed(serialo, filename, ctrltext='x', delay=1):
    """Flash programming with serial based on xmodem protocl.

    Arguments:
        serialo {serial.Serial object} -- an serial.Serial object
        filename {str} -- path to file.
        ctrltext {str} -- the string will be written to serial to control the loader mode.
                        default is 'x'
		delay {number} -- delay time in second

    Exceptions:
        SerialCannotOpenException -- raise when serial cannot be open

    Returns:
        boolean -- There is no error, it will return True, else False.
    """

    if not serialo.is_open:
        serialo.open()

    cnt, dat = 0, ""
    logging.info("checking serial output")
    delay = delay + 10
    # read data when 'READY' is in the context, or timeout.
    while True:
        n = serialo.inWaiting()
        dat = serialo.read(n)

        # logging the serial ouput
        if dat != "":
            logging.info("\n" +dat)

        # check string
        if "READY" in dat:
            break

        # check timeout
        if cnt > delay:
            logging.warning("Timeout: not found 'READY' in serial output..")
            break

        cnt = cnt + 1
        time.sleep(1)

    logging.info("delay %s s", delay)
    #time.sleep(delay)
    def getc(size, timeout=10):
        return serialo.read(size)


    def putc(data, timeout=10):
        sys.stdout.write('=')
        serialo.write(data, log=False)
        time.sleep(0.001)

    serialo.write(ctrltext)
    logging.info("> Download program via XMODEM protocol: %s", filename)

    result = False
    with open(filename, 'rb') as stream:
        modem = XMODEM(getc, putc)
        result = modem.send(stream, retry=4)
        serialo.close()

    if result:
        logging.info("Flash successful")
    else:
        logging.warning("Flash failure!!!")

    return result





import os
import glob
import logging
import tempfile

from difflib import SequenceMatcher
from xml.etree import cElementTree as ET
from distutils.version import LooseVersion

from mcutk.apps import eclipse
from mcutk.apps.projectbase import ProjectBase
from mcutk.exceptions import ProjectNotFound, InvalidProject, ProjectParserError


class Project(eclipse.Project):
    """MCUXpresso SDK and projects parser tool."""

    PROJECT_EXTENSION = '*.xml'


    @classmethod
    def frompath(cls, path):
        """Return a project instance from a given file path or directory.

        If path is a directory, it will search the project file and return an instance.
        Else this will raise mcutk.apps.exceptions.ProjectNotFound.
        """
        if not os.path.exists(path):
            raise ProjectNotFound("No such file or directory: %s", path)

        if os.path.isfile(path):
            return cls(path)

        instance = None
        for file in glob.glob(path + "/.project"):
            with open(file) as f:
                content = f.read()
                if '<name>mcux</name>' in content or 'mcuxpresso' in content:
                    instance = cls(file)
                    break

        if instance:
            return instance

        for file in glob.glob(path + "/*" + cls.PROJECT_EXTENSION):
            try:
                instance = cls(file)
                break
            except InvalidProject:
                pass
        else:
            raise ProjectNotFound("Not found <example_app>.xml")

        return instance



    def __init__(self, prjpath, sdk_root=None, **kwargs):
        """MCUXPressoIDE project constructor.

        Arguments:
            prjpath {str} -- path to <name>.xml

        Keyword Arguments:
            sdk_root {str} -- path to sdk package root, default {None} that will be loaded from xml.
        """
        super(Project, self).__init__(prjpath, **kwargs)
        self._is_package = False
        self._sdk_root = sdk_root
        self._name = ''
        self._targets = None
        self._sdkmanifest = None
        self._example_id = None
        self._nature = 'org.eclipse.cdt.core.cnature'

        # eclipse project
        self._is_package = not (prjpath.endswith('.project') or prjpath.endswith('.cproject'))

        if self._is_package:
            self._load_from_sdk_package(prjpath)
            self._properties_init()
        else:
            self._load_from_eclipse_project(prjpath)



    @property
    def is_enabled(self):
        if not self.is_package:
            return True

        # check manifest: this exmaple is enabled for mcux
        example_info = self.sdkmanifest.find_example(self._example_id)
        return "mcux" in example_info.get("toolchain", "")




    @property
    def sdkmanifest(self):
        return self._sdkmanifest



    @sdkmanifest.setter
    def sdkmanifest(self, value):
        if not isinstance(value, SDKManifest):
            raise ValueError("Must be a SDKManifest object")

        self._sdkmanifest = value





    @property
    def is_package(self):
        """Package project or standard eclipse project"""
        return self._is_package





    def _load_from_eclipse_project(self, path):
        """Load from Eclipse C/C++ project"""
        self.parse(path)




    def _load_from_sdk_package(self, path):
        """Load from SDK <app>.xml and *_manifest*.xml.
            1. Parse <app>.xml to get manifest.xml,
            2. Get related information from manifest.
        """

        self._targets = self._conf.keys()

        xmlroot = ET.parse(path).getroot()
        example_node = xmlroot.find('./example')
        if example_node is None:
            raise InvalidProject('Unable to find <example> node.')

        self._example_id = example_node.attrib.get('id')
        self._name = example_node.attrib.get('name')

        try:
            self._nature = example_node.find('projects/project[@nature]').attrib.get('nature')
        except:
            pass

        if not self._example_id:
            raise ProjectParserError('None id in exmaple node! %s'%self.prjpath)


        self._conf = {
            'Debug': self._example_id + '/Debug/',
            'Release': self._example_id + '/Release/'
        }


        if self.sdkmanifest:
            return


        # Automaticlly find and load SDKManifest
        prjdir_abs = os.path.abspath(self.prjdir).replace('\\', '/')

        def _search_node(node):
            """get sdk_root from an XML element node."""
            if node is None:
                return

            source_path = node.attrib.get('path')
            if not source_path:
                return

            if source_path in prjdir_abs:
                sdk_root = prjdir_abs.replace(source_path, "")
            else:
                match = SequenceMatcher(None, prjdir_abs, source_path).find_longest_match(0, len(prjdir_abs), 0, len(source_path))
                sdk_root = prjdir_abs[:match.a]

            if os.path.exists(sdk_root):
                return sdk_root
            return


        def _search_possible_nodes():
            possible_nodes = [
                './source[@type="src"]',
                './source'
            ]

            for p in possible_nodes:
                for node in example_node.findall(p):
                    sdk_root = _search_node(node)

                    if not sdk_root:
                        continue

                    try:
                        return SDKManifest.load_from_dir(sdk_root)
                        break
                    except ProjectParserError:
                        pass

        def _search_from_local():
            # In some situation, example.xml not include the source element.
            # Added a workaround to find mainfest file in it's parent.
            current_dir = prjdir_abs
            while True:
                parent_dir = os.path.dirname(current_dir)
                # system root
                if parent_dir == current_dir:
                    break
                try:
                    return SDKManifest.load_from_dir(parent_dir)
                except ProjectParserError:
                    pass

                current_dir = parent_dir


        manifest = _search_possible_nodes()

        if not manifest:
            manifest = _search_from_local()

        if manifest:
            self.sdkmanifest = manifest
        else:
            raise ProjectParserError("Unable to find SDK Manifest!")





    def _properties_init(self):
        """Init build properties variable.

        sdk.location = D:/Users/B46681/Desktop/SDK_2.0_MK64FN1M0xxx12-drop4
            This is the location where your SDK have been downloaded.
            You can use either zip or folder containing the SDK
            Please remember that if you want to create linked resources into your project(i.e. standalone = false) you need to use a folder instead of a zip.
            NOTE: on Windows you have to use "//" or "/".

        example.xml = D:/Users/B46681/Desktop/SDK_2.0_MK64FN1M0xxx12-drop4/boards/frdmk64f/demo_apps/hello_world/mcux/hello_world.xml
            If adding the "example.xml" property, the examples are retrieved from that specific file and shall valid against the used SDK
            NOTE: on Windows you have to use "//" or "/".

        nature = org.eclipse.cdt.core.cnature
            This represents the nature of your project (i.e. C or C++)
            It can be:
                - org.eclipse.cdt.core.cnature for C projects
                - org.eclipse.cdt.core.ccnature for C++ projects
                (Please remember that the example your're going to create shall support the C++ nature)

        standalone = true
            If true, it will copy the files from the SDK, otherwise it will link them.
            Note: linked resources will be only created if the SDK is provided as a folder

        project.build = true
            If true, the project will be compiled, otherwise the project is only created.

        clean.workspace = true
            True, if you want to clear the workspace used, false otherwise

        build.all = false
            If true, all the examples from all the SDK will be created, otherwise you need specify the SDK name

        skip.default = false
            If true, skip the default SDKPackages folder and all its content
            Default is false

        sdk.name = SDK_2.0_MK64FN1M0xxx12
            The SDK name (i.e. the folder/file name without extension)
            NOTE: only used when build.all = false

        board.id = frdmk64f
            The board id as for the manifest definition
            NOTE: only used when build.all = false

        Other Settings:
            verbose = true
                If true, more info will be provided using stdout

            indexer = false
                If true, enable the CDT indexer, false otherwise

            project.build.log = true
                If true, show the CDT build log, false otherwise

            simple.project.name = true

        """
        self._buildproperties = {
            'sdk.location': None,
            'example.xml': None,
            'nature': 'org.eclipse.cdt.core.cnature',
            'standalone': 'true',
            'project.build': 'true',
            'clean.workspace': 'true',
            'build.all': 'false',
            'build.config': 'debug',
            'simple.project.name': 'false',
            'use.other.files': 'true ',
            'skip.default': 'true',
            'sdk.name': None,
            'board.id': '',
            'verbose': 'false',
            'indexer': 'false',
            'use.io.console': 'false',
            'project.build.log': 'true'
        }


    def gen_properties(self, target, dir=None):
        """Return a file path for properties file.

        Arguments:
            target -- {string} target configuration
            dir -- {string} the location to place the new geneated file, default is system tempfile.

        """
        # boardid will effect workspace path
        board_ids = self.sdkmanifest.boards
        boardid = self._example_id.replace("_" + self._name, '')
        if boardid not in board_ids:
            boardid = board_ids[0]

        logging.info("SDK Manifest Version: %s", self.sdkmanifest.manifest_version)

        self.setproperties("example.xml", self.prjpath.replace('\\', '/'))
        self.setproperties("sdk.location", self.sdkmanifest.sdk_root.replace('\\', '/'))
        self.setproperties("nature", self.nature)
        self.setproperties("sdk.name", self.sdkmanifest.sdk_name)
        self.setproperties("board.id", boardid)
        self.setproperties("build.config", target)


        with tempfile.NamedTemporaryFile(dir=None, delete=False, prefix="mcuxpresso_", mode='w') as f:
            for per_property, value in self._buildproperties.items():
                f.writelines("{0} = {1}\r\n".format(per_property, value))
            properties_file = f.name

        logging.debug('properties file: %s', properties_file)
        return properties_file



    def setproperties(self, attrib, value):
        """ Set the value of self._buildproperties"""

        self._buildproperties[attrib] = value



    @property
    def nature(self):
        return self._nature



    @property
    def targets(self):
        """Return all targets name

        Returns:
            list -- a list of targets
        """
        if self._targets:
            return list(self._targets)
        else:
            return ['Debug', 'Release']


    @property
    def name(self):
        """Return the application name

        Returns:
            string --- app name
        """
        return self._name








class SDKManifest(object):
    """NXP MCUXpresso SDK Manifest Parser."""


    @classmethod
    def load_from_dir(cls, sdk_root):
        """Load latest version of manifest from directory."""

        manifestfilelist = glob.glob("{0}/*_manifest*.xml".format(sdk_root))
        if not manifestfilelist:
            raise ProjectParserError("cannot found manifest file")

        file_versions = {}
        for per_file in manifestfilelist:
            version_str = per_file.replace('.xml', '').split('_manifest')[-1]
            version =  version_str[1:] if version_str.startswith('_') else version_str
            if version:
                file_versions[version] = per_file

        ver_latest = sorted(file_versions.keys(), key=lambda v: LooseVersion(v))[-1]
        manifest_path = file_versions[ver_latest].replace("\\",'/')

        return SDKManifest(manifest_path)




    def __init__(self, filepath):
        xmlParser = ET.parse(filepath)
        self._xmlroot = xmlParser.getroot()
        self._sdk_root = os.path.dirname(filepath)
        self._manifest_version = self._xmlroot.attrib['format_version']
        self._sdk_name = self._xmlroot.attrib["id"]
        self._sdk_version = self._xmlroot.find('./ksdk').attrib['version']

    @property
    def sdk_version(self):
        return self._sdk_version



    @property
    def sdk_name(self):
        return self._sdk_name



    @property
    def manifest_version(self):
        return self._manifest_version


    def find_example(self, example_id):
        """Return a dict which contain exmaple attributes.

        Keys:
            - id
            - name
            - toolchain
            - brief
            - category
            - path
        """
        xpath = './boards/board/examples/example[@id="{0}"]'.format(example_id)
        node = self._xmlroot.find(xpath)
        if not node:
            raise Exception("Cannot found example in manifest, id: %s", example_id)

        return node.attrib

    @property
    def boards(self):
        xpath = './boards/board'
        nodes = self._xmlroot.findall(xpath)
        return [n.attrib['id'] for n in nodes]



    @property
    def sdk_root(self):
        return self._sdk_root









import os
import sys
import re
import glob
import platform
import subprocess
import logging
import threading
import time
import tempfile
from mcutk.debugger.base import DebuggerBase

PY = sys.version_info[0]



class JLINK(DebuggerBase):
    """J-Link debugger is a wrapper for SEGGER-JLink.
    """

    @staticmethod
    def get_latest():
        """Get latest installed instance from the system.
        """
        return _scan_installed_instance()



    def __init__(self, *args, **kwargs):
        """Create an instance of the JLink debugger class.
        """
        super(JLINK, self).__init__("jlink", *args, **kwargs)
        osname = platform.system().lower()
        if osname == "windows":
            self._jlink_exe = os.path.join(self.path, "JLink.exe")
            self._jlink_gdbserver_exe = os.path.join(self.path, "JLinkGDBServerCL.exe")
        else:
            self._jlink_exe = os.path.join(self.path, "JLinkExe")
            self._jlink_gdbserver_exe = os.path.join(self.path, "JLinkGDBServerCLExe")


    @property
    def is_ready(self):
        return os.path.exists(self._jlink_exe)



    def _run_jlink_exe(self, jlink_exe_cmd, timeout_sec):
        """Run jlink.exe process.

        Arguments:
            jlink_exe_cmd {list or string} -- JLink.exe or JLinkExe command line
            timeout_sec {int} -- Max timeout in seconds

        Returns:
            Tuple -- (retcode, output)
        """
        def timeout_exceeded(p):
            p.kill()
            raise Exception('JLink process:%s exceeded timeout!', p.pid)

        kwargs = {
            "stdout": subprocess.PIPE,
            "stderr": subprocess.STDOUT,
        }
        if PY > 2:
            kwargs['encoding'] = 'utf8'

        process = subprocess.Popen(jlink_exe_cmd, **kwargs)
        if timeout_sec is not None:
            timeout = threading.Timer(timeout_sec, timeout_exceeded, [process])
            timeout.start()

        # Grab output of JLink.
        output, _ = process.communicate()
        if timeout_sec is not None:
            timeout.cancel()

        logging.debug('JLink response: {0}'.format(output))
        return process.returncode, output




    def run_script(self, filename, timeout_sec=60):
        """Run jlink script with JLink.exe with a timeout timer.

        Arguments:
            filename -- {string} path to jlink script
            timeout_sec -- {int} seconds for timeout value, default 60s.

        Returns:
            tuple -- (jlink_exit_code, console_output)
        """

        if self._board:
            jlink_exe_cmd = [
                self._jlink_exe,
                "-SelectEmuBySN",
                "{0}".format(self._board.usbid),
                "-Device",
                "{0}".format(self._board.devicename),
                "-IF",
                "{0}".format(self._board.interface),
                "-Speed",
                "auto",
                '-autoconnect',
                '1',
                "-ExitOnError",
                "-CommanderScript",
                filename
            ]
            # default jtag chain
            if self._board.interface.upper() == "JTAG":
                jlink_exe_cmd.append("-jtagconf")
                jlink_exe_cmd.append("-1,-1")

        else:
            jlink_exe_cmd = [
                self._jlink_exe,
                "-SelectEmuBySN",
                "6210000",
                "-CommanderScript",
                filename
            ]



        return self._run_jlink_exe(jlink_exe_cmd, timeout_sec)




    def run_commands(self, commands, timeout_sec=60):
        """Run a list of commands by JLink.exe.

        Arguments:
            commands -- {list} list of JLink.exe commands
            timeout_sec -- {int} seconds for timeout value
        """
        # Create temporary file to hold script.
        script_file = tempfile.NamedTemporaryFile(mode='w', delete=False)
        commands = '\n'.join(commands)
        script_file.write(commands)
        script_file.close()
        logging.debug('Using script file name: {0}'.format(script_file.name))
        logging.debug('Running JLink commands: {0}'.format(commands))

        return self.run_script(script_file.name, timeout_sec)


    def test_conn(self):
        """Test debugger connection."""

        p1 = re.compile("Connecting to J-Link via USB.{3}FAILED")
        p2 = re.compile("Found.*JTAG")
        p3 = re.compile("Found.*SW")
        p4 = re.compile("JTAG chain detection found 1 devices")

        commands = [
            'regs',
            'qc'
        ]
        # Run commands.
        _, output = self.run_commands(commands, timeout_sec=30)
        if p1.search(output) is not None:
            return "NotConnected"
        elif (p2.search(output) is not None) or (p3.search(output) is not None) \
            or (p4.search(output) is not None):
            return "NoError"
        else:
            return "Error"


    def erase(self):
        """Erase flash."""
        commands = [
            'r',      # Reset
            'erase',  # Erase
            'r',      # Reset
            'q'       # Quit
        ]
        return self.run_commands(commands)


    def unlock(self):
        """Unlock kientis device."""

        commands = [
            'unlock Kinetis',
            'q'
        ]
        return self.run_commands(commands)



    def reset(self):
        """Reset chip and not halt."""
        commands = [
            'rnh',      # Reset
            'q'         # Quit
        ]
        # Run commands.
        return self.run_commands(commands, timeout_sec=30)

    def to_hex(self, Val):
        if isinstance(Val, int):
            Val = hex(Val)
        elif isinstance(Val, long):
            Val = hex(Val).replace("L", '')
        else:
            pass
        return Val


    def write_reg32(self, regAddr, writeValue):
        """
        Write 32-bit register by JLink command line.
        """
        commands = ["w4 {0}, {1}".format(self.to_hex(regAddr), self.to_hex(writeValue)),        # Write 32-bit register
                    "qc"        # Quit
        ]

        # logging.info('Write 32-bit register:\n%s'%output)
        msg = 'Write 32-bit register...\n'\
              '================================\n'\
              'Writing {0} -> {1}\n'\
              '================================\n'.format(self.to_hex(writeValue), self.to_hex(regAddr))
        logging.info(msg)

        # Run commands.
        return self.run_commands(commands, timeout_sec=30)

    def read_reg32(self, regAddr):
        """
        Read 32-bit register by JLink command line.
        """
        commands = ["mem {0}, 0x4".format(self.to_hex(regAddr)),       # read 32-bit register
                    "qc"        # Quit
        ]
        returncode, output = self.run_commands(commands, timeout_sec=30)

        pattern = self.to_hex(regAddr).replace("0x", '')
        matchObj = "(.*){0} = (.*) ".format(pattern)

        regVal = 'Unknown'

        for line in output.splitlines():
            if re.match(matchObj, line, re.M) == None:
                continue
            else:
                regValList = re.match(matchObj, line, re.M).group(2).split()
                regValList.reverse()
                regVal = '0x' + ''.join(regValList)
        return (returncode, regVal)

    def flash(self, filepath, addr=0):
        """Program binary to flash.
        The file could be ".bin" or ".hex". addr is the start address.
        """
        address = addr
        if self._board.start_address:
            address = self._board.start_address

        # Build list of commands to program hex files.
        commands = ['r']   # Reset

        # Program each hex file.
        if filepath.endswith(".hex"):
            commands.append('loadfile "{0}"'.format(filepath))

        elif filepath.endswith(".bin"):
            commands.append('loadbin "{0}" {1}'.format(filepath, address))

        commands.extend([
            'r',  # Reset
            'g',  # Run the MCU
            'q'   # Quit
        ])
        logging.info("flash start address: %s", address)
        # call registerd callback function
        self._call_registered_callback("before-load")
        # Run commands.
        return self.run_commands(commands)


    def list_connected_devices(self):
        """Return a list of connected id list."""

        devices = list()
        ret, raw_data = self.run_commands([ "ShowEmuList", "qc"], 10)
        if ret != 0:
            return devices

        reg1 = re.compile(r"number: -{0,1}\d{5,15}, ProductName:")
        reg2 = re.compile(r"-{0,1}\d{5,15}")

        for line in raw_data.split('\n'):
            if reg1.search(line)is not None:
                m = reg2.search(line)
                if m is not None:
                    usb_id = m.group(0)
                    if '-' in usb_id:
                        usb_id = str(0xFFFFFFFF + int(usb_id) + 1)
                    devices.append({'debugger': 'jlink', "type": 'jlink', 'usbid': usb_id})

        return devices



    def get_gdbserver(self, **kwargs):
        """Return gdbserver startup shell command.

        Example returns:
            JLinkGDBServerCL.exe -if <JTAG/SWD> -speed auto -device <device name> -port <port>
            --singlerun -strict -select usb=<usb serial number>
        """

        board = kwargs.get('board')
        speed = kwargs.get('speed', 'auto')
        interface = kwargs.get('interface', 'SWD')
        port = kwargs.get('port')
        usbid = kwargs.get('usbid')
        jlinkscript = kwargs.get('jlinkscript')
        devicename = kwargs.get('devicename')


        if board is None:
            board = self._board

        if board and board.gdbport:
            devicename = board.devicename
            interface = board.interface
            port = board.gdbport
            usbid = board.usbid

        if not devicename:
            logging.warning('jlink: not configure device name.')

        options = "-if {} -singlerun -strict".format(interface, devicename)

        if devicename:
            options += " -device %s"%devicename

        if usbid not in [None, ""]:
            options = "-select usb={} {}".format(usbid, options)

        if port:
            options += " -port %s"%port

        if speed:
            options += " -speed %s"%speed

        if jlinkscript:
            options += " -jlinkscriptfile " + jlinkscript

        return "\"%s\" %s"%(self._jlink_gdbserver_exe, options)



    def gdb_init_template(self):
        """Defined default gdb commands for J-Link gdb-server."""

        commands = [
            "set tcp connect-timeout 10",
            "set remotetimeout 10",
            "target remote localhost: {gdbport}",
            "monitor speed auto",
            "monitor reset 0",
            "monitor halt",
            "load",

        ]

        if self._board and self._board.sp:
            commands.append("monitor reg sp = {sp}")

        if self._board and self._board.pc:
            commands.append("monitor reg pc = {pc}")

        commands.append("monitor go")
        commands.append("q")

        return "\n".join(commands)







def _scan_installed_instance():
    osname = platform.system().lower()
    if osname == "windows":
        try:
            import _winreg as winreg
        except ImportError:
            import winreg

        try:
            root_entry = "SOFTWARE\WOW6432Node\SEGGER\J-Link"
            jlink_key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, root_entry, 0, winreg.KEY_READ)
            path = winreg.QueryValueEx(jlink_key, "InstallPath")[0]
            version = winreg.QueryValueEx(jlink_key, "CurrentVersion")[0]
            winreg.CloseKey(jlink_key)
            return JLINK(path, version=version)
        except WindowsError:
            return None

    elif osname == 'linux':
        default_path = "/opt/SEGGER"
        jlinks = glob.glob(default_path + "/JLink*")
        if jlinks:
            jlinks.sort()
            path = jlinks[-1]
            version = path.split("JLink")[-1].replace('_', '')
            return JLINK(path, version=version)
        return None



from __future__ import print_function
import click
import glob
import os
import logging
from os.path import join

from mcutk.debugger import getdebugger
from mcutk.managers.conf_mgr import ConfMgr




class Debughelper(object):

    supported_debuggers = ['jlink', 'pyocd']


    @staticmethod
    def list_debuggers():
        """List installed debuggers from system."""

        debs = list()
        for name in Debughelper.supported_debuggers:
            debugger = getdebugger(name).get_latest()
            if debugger and debugger.is_ready:
                debs.append(debugger)
            else:
                click.secho('warning: cannot found %s installation, please check if it is installed!'%name, fg="yellow")

        return debs



    @staticmethod
    def list_devices():
        """List connected devices."""

        debuggers = Debughelper.list_debuggers()
        devices = list()
        for deb in debuggers:
            for item in deb.list_connected_devices():
                item['debugger'] = deb
                devices.append(item)

        return devices


    @staticmethod
    def choose_device(device):
        """Promt user to select device."""

        if device == None:
            device = dict()

        debugger = None

        debugger_type = device.get('debugger_type')
        usbid = device.get('usbid')

        devices = Debughelper.list_devices()

        if len(devices) == 0:
            click.secho('no devices found!', fg='red')
            return None, None


        selected_device = None
        # use usb id to find debugger type
        if usbid:
            for d in devices:
                if d['usbid'] == str(usbid):
                    selected_device = d
                    break
            else:
                logging.warning('not found usbid in system, is it right?', fg='red')
                return None, None

        else:
            # prompt user to select
            if devices and len(devices) == 1:
                selected_device = devices[0]
            else:
                for index, item in enumerate(devices):
                    print('{:2} - {:10} - {} - {}'.format(index, item['type'], item.get('name'), item['usbid']))

                while True:
                    try:
                        index = raw_input('Please input the index to select device > ')
                        index = int(index)
                        selected_device = devices[index]
                        break
                    except (ValueError, IndexError) as e:
                        pass

        device['usbid'] = selected_device['usbid']
        device['debugger_type'] = selected_device['type']



        if device.get('debugger_type') == 'jlink':
            if not device.get('devicename'):
                device['devicename'] = raw_input('Please input device name > ')

            if not device.get('interface'):
                interface = raw_input('SWD or JTAG > ')
                device['interface'] = "SWD" if not interface else interface

        debugger = selected_device.pop('debugger')
        return debugger, device






import abc
import os
import signal
import sys
import time
import logging
import subprocess
import tempfile
import threading
import socket
import errno
import shlex

import mbed_lstools
import pexpect
from pexpect.popen_spawn import PopenSpawn
from mcutk.appbase import APPBase
from mcutk.gdb_session import GDBSession


class DebuggerBase(APPBase):
    __metaclass__ = abc.ABCMeta

    def __init__(self, *args, **kwargs):
        super(DebuggerBase, self).__init__(*args, **kwargs)
        self.gdbpath = kwargs.get("gdbpath", "")
        self.version = kwargs.get("version", "unknown")
        self._board = None
        self._callback_map = {
            "before-load": None
        }


    def set_board(self, board):
        self._board = board





    def gdb_init_template():
        """Return a string about gdb init template.
        """
        return ""




    @abc.abstractmethod
    def reset(self):
        """Used to reset target CPU.
        """
        pass


    @abc.abstractmethod
    def erase(self):
        """Used to erase flash.
        """
        pass


    @abc.abstractmethod
    def flash(self):
        """Binary image programming.
            .bin
            .hex
        """
        pass



    def get_gdbserver(self, board=None):
        """Return a string about the command line of gdbserver
        """
        pass



    def start_gdbserver(self, **kwargs):
        gdbserver_cmd = self.get_gdbserver(**kwargs)
        print (gdbserver_cmd)
        # start gdb server
        retcode = subprocess.call(gdbserver_cmd, shell=True)



    def list_connected_devices(self):
        mbeds = mbed_lstools.create()
        devices = mbeds.list_mbeds()
        for device in devices:
            device['usbid'] = device.pop('target_id_usb_id')
            device['type'] = device.pop('device_type')
            device['name'] = device.pop('platform_name')
            if device['type'] == 'daplink':
                device['debugger'] = 'pyocd'

        return devices


    def gdb_program(self,
                    filename,
                    gdbserver_cmdline=None,
                    gdbinit_commands=None,
                    board=None,
                    timeout=200,
                    **kwargs):
        """Using gdb & gdbserver to programming image.
        Steps:
            1> Start gdbserver at port: board.gdbport
            2> Render gdbinit_template
            3> Start gdb.exe:
                gdb.exe -x <gdb.init> -se <binary.file>

        Arguments:
            filename - {str}: path to image file.
            gdbserver_cmdline - {str}: gdb server command line, used for starting gdb server.
            gdbinit_commands - {str}: gdb init commands to control gdb behaviour.
            timeout - {int}: set timeout for gdb & gdb server process. default 200 seconds.

        Returns:
            tuple --- (returncode, console-output)
        """
        if board is None:
            board = self._board

        if board is None:
            raise ValueError('no board is associated with debugger!')

        # load gdb init template
        gdb_init_template = board.gdb_init_commands
        if gdb_init_template is None:
            gdb_init_template = self.gdb_init_template()

        gdbcommands = gdbinit_commands if gdbinit_commands else \
                      render_gdbinit(gdb_init_template, board)

        gdbserver_cmdline = gdbserver_cmdline if gdbserver_cmdline else self.get_gdbserver(**kwargs)

        return self._gdb_program(board.gdbport,
                                 filename.replace("\\", "/"),
                                 gdbserver_cmdline,
                                 gdbcommands,
                                 timeout)


    def _gdb_program(self, gdbport, filename, gdbserver_cmd,
                    gdbinit, timeout):
        """use gdb to load program to target device"""

        def _kill_popen_process(process):
            # process.kill() just killed the parent process, and cannot kill the child process
            # that caused the popen process in running state.
            # force to use windows command to kill that the process!
            if not isinstance(process, subprocess.Popen):
                raise TypeError("argument is not a instance of subprocess.Popen")

            if os.name == "nt":
                os.system("TASKKILL /F /PID {pid} /T".format(pid=process.pid))
            else:
                process.kill()


        def timeout_exceeded(ps):
            """subprocess tiemout exceeded handler."""
            for p in ps:
                if isinstance(p, PopenSpawn):
                    p.kill(None)
                else:
                    _kill_popen_process(p)
                logging.warning('pid: %s exceeded timeout!', p.pid)



        logging.info("> starting gdb server.")
        logging.info(gdbserver_cmd)
        if os.name != "nt":
            gdbserver_cmd = shlex.split(gdbserver_cmd)

        # start gdb server
        gdbserverPro = subprocess.Popen(
            gdbserver_cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            shell=False,
            )

        if not _validate_port_is_ready(gdbserverPro, gdbport):
            logging.error("gdb server start failure")
            if gdbserverPro.poll() is None:
                _kill_popen_process(gdbserverPro)
            output, _ = gdbserverPro.communicate()
            logging.error(">>>> gdb server console output: \n\n%s", output)
            return 1, output

        logging.info("> gdb server is ready, pid: {0}, port: {1}".format(gdbserverPro.pid, gdbport))
        gdb_output = list()

        def stdout_reader(process):
            for line in iter(process.stdout.readline, b''):
                gdb_output.append(line)

        # To resolve large output from subprocess PIPE, use a background thread to continues read
        # data from stdout.
        reader_thread = threading.Thread(target=stdout_reader, args=(gdbserverPro, ))
        reader_thread.start()

        # start gdb client
        output = ""
        gdb_errorcode = 0

        logging.info("> start gdb client to connect to server: localhost, port: %s", gdbport)

        gdb_cmd = "{} --exec {} --silent".format(self.gdbpath, filename)
        session = GDBSession.start(gdb_cmd)

        # set timeout
        # Use a timer to stop the subprocess if the timeout is exceeded.
        if timeout is not None:
            ps_list = [gdbserverPro, session]
            process_timer = threading.Timer(timeout, timeout_exceeded, (ps_list, ))
            process_timer.start()

        # convert string commands to a list
        _gdb_actions = [line.strip() for line in gdbinit.split("\n") if line.strip()]

        for act in _gdb_actions:
            # call registerd callback function before-load command
            if act.startswith("load"):
                self._call_registered_callback("before-load")

            try:
                c = session.run_cmd(act).lower()
                if "No connection could be made" in c:
                    gdb_errorcode = 1
                    logging.error(c)
                    break
                elif '"monitor" command not supported by this target' in c:
                    gdb_errorcode = 1
                    logging.error('gdb command execute error!')
                    break
            except:
                logging.exception('gdb command run error, CMD: %s', act)

        # gdb client disconnect the connection,
        # and gdbsever will automaticlly close
        session.close()
        gdbserverPro.wait()
        reader_thread.join()

        # Stop timeout timer when communicate call returns.
        if timeout is not None:
            process_timer.cancel()

        logging.info("> gdb server exit! exit code: %s", gdbserverPro.returncode)

        # get gdb console output
        output = ''.join(gdb_output)
        output += session.console_output

        if gdb_errorcode == 0:
            retcode = gdbserverPro.returncode
        else:
            retcode = gdb_errorcode

        return retcode, output




    def register(self, name):
        """Declare a decorator to register callback to debugger instance.

        Arguments:
            name {str} -- before-load
        """
        def func_wrapper(func, *args, **kwagrs):
            self._callback_map[name] = (func, args, kwagrs)
            return func
        return func_wrapper


    def _call_registered_callback(self, name=None):
        value = self._callback_map.get(name)
        if type(value) is tuple:
            func, args, kwargs = value
            if func:
                return func(*args, **kwargs)

        return None



def render_gdbinit(template, board):
    """
    Render gdbinit template with board object.

    Render used '.foramt()' syntax:
        'target remote localhost: {gdbport}'

    Example:
        1. jlink

    """
    dicta = board.__dict__
    # dicta["file"] = executable
    return template.format(**dicta)





def _validate_port_is_ready(server_process, port, tiemout=30):
    """Validate the port is open on localhost"""

    is_ready = False
    port = int(port)
    s = None

    assert server_process != None

    # delay 1 seconds wait server up
    time.sleep(1)
    for _ in range(tiemout):
        print(" Wait for gdb server ready.")
        time.sleep(0.5)

        if server_process.poll() is None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                s.bind(("127.0.0.1", port))
            except socket.error as e:
                if e.errno == errno.EADDRINUSE:
                    is_ready = True
                    break
                else:
                    print(e)
        else:
            break

        if s is not None:
            s.close()

    return is_ready




from __future__ import print_function
import os
import csv
import sys
import click
import glob
import logging
import platform
from os.path import join

from mcutk.apps import appfactory
from mcutk.exceptions import ProjectNotFound
from mcutk.projects_scanner import find_projects, TOOLCHAINS

# from ._settings import TOOLCHAINS, EXCLUDE_DIR_NAME
from mcutk.managers.conf_mgr import ConfMgr


def _get_ides(ides):
    # apps conifguration from config file
    ide_apps = dict()
    os_name = platform.system()
    get_app = ConfMgr.load().get_app

    for tool_name in ides:
        info = get_app(tool_name)
        tool_class = appfactory(tool_name)

        app = None
        if info:
            app = tool_class(**info)
        elif os_name in tool_class.OSLIST:
            app = tool_class.get_latest()

        if app and app.is_ready:
            ide_apps[tool_name] = app
        else:
            ide_apps[tool_name] = None

    return ide_apps


def _project_operation(app, project, target, log, list, recursive):
    target = project.map_target(target)
    msg = '  => {}'.format(target)

    click.echo(msg)
    ret = app.build_project(project, target, log)
    color_table = { 0: "green", 1: "red", 2: "yellow" }
    color = color_table.get(ret.value)

    if log:
        click.echo('     - LOG: %s'%log)

    click.secho('     - BUILD %s'%ret.name, fg=color)
    if ret.value in (0, 2):
        click.echo('     - OUTPUT %s'%ret.output)

    return ret.name


def _dump_to_csv(data):
    keys = data[0].keys()
    csvfile = 'mcutk_build.csv'
    print ('dumped to file %s'%csvfile)
    with open(csvfile, 'w') as output_file:
        dict_writer = csv.DictWriter(output_file, keys)
        dict_writer.writeheader()
        dict_writer.writerows(data)





@click.command('build',
    short_help='find and build project',
    help="build projects with specific path",
    epilog="supported toolchains: %s"%TOOLCHAINS)
@click.argument('path', default=".", type=click.Path(exists=True))
@click.option('-t', '--target', help='set target name, if not specificed, the first known target is used')
@click.option('-l', '--log', type=click.Path(), help='specifies the output log file')
@click.option('-v', '--verbose', is_flag=True, help='show more console message')
@click.option('--no-build', 'no_build', is_flag=True, help='list project information only, not execute build!')
@click.option('-r', '--recursive', is_flag=True, default=False, help='recursive the directory and build all configuration')
@click.option('--include-tools', 'include_tools', nargs=1, help='specific toolchains, multiple value need to join with \",\", recursive mode only')
@click.option('--exclude-tools', 'exclude_tools', nargs=1, help='exlcude toolchains, multiple value need to join with \",\", recursive mode only')
@click.option('--results-csv', 'result2csv', is_flag=True, help='dump results to CSV format')
def cli(path, target, log, verbose, no_build, recursive, include_tools, exclude_tools, result2csv):
    """Find and build projects"""
    if recursive:
        print("recursive mode: true, log will be saved to project root directory.")

    if include_tools:
        include_tools = include_tools.split(',')
        print("include-tools: %s"%str(include_tools))

    if exclude_tools:
        exclude_tools = exclude_tools.split(',')
        print("exclude-tools: %s"%str(exclude_tools))

    project_root = path
    projects, count = find_projects(project_root, recursive, include_tools, exclude_tools)

    if not projects:
        click.secho('Error: not found projects', fg='red', err=True)
        return

    click.echo("Toolchains configuration: ")

    ide_apps = _get_ides(projects.keys())
    for name, ideapp in ide_apps.items():
        if ideapp:
            click.secho(' + {0:<10} | v={1:<20} | p=\"{2}\"'.format(ideapp.name, ideapp.version, ideapp.path), fg="green")
        else:
            click.echo(" - Warning: %s is unavaliable."% name)

    click.echo("\n============================= build session starts =============================")

    results = list()
    c = 1

    for toolname, plist in projects.items():
        for project in plist:
            click.echo('[{}/{}] Building {} - {}, project: {}'.format(c, count, project.idename, project.name, project.prjpath))
            c += 1
            if no_build:
                click.echo(' avaliable targets')
                for tname in project.targets:
                    click.echo('  - %s'% tname)
                continue

            if target:
                targets_to_build = [target]
            elif recursive and not target:
                targets_to_build = project.targets
            else:
                targets_to_build = [project.targets[0]]


            for target_name in targets_to_build:
                ideapp = ide_apps.get(project.idename)
                if not ideapp:
                    click.secho('Error: %s have no configuration, skip'%project.idename, fg='red')
                    continue

                if recursive:
                    log = os.path.join(project.prjdir, "build_" + target_name.replace(" ", "_") + ".log").replace('\\', '/')

                r = _project_operation(ideapp, project, target_name, log, list, recursive)
                ret = {
                    "name": project.name,
                    "toolchain": project.idename,
                    "target": target_name,
                    "result": r,
                    "log": log
                }

                results.append(ret)

    if result2csv:
        _dump_to_csv(results)




from __future__ import print_function
import logging
from mcutk.board import Board


class BoardUnion(object):
    """BoardUnion: unite boards to a group.

    Boards are involved in a group, each board has an alias.
    BoardUnion make the boards relationship more clearly.

    """
    def __init__(self, boards=None, id=None, name="union",  **kwargs):
        """Create a BoardUnion instance.

        Arguments:
            boards {list or dict} -- create a unio by  list of boards or a boards dict.
            id {string} -- union id, default: None
            name {string} -- the name of this union, default: union
        """
        self.id = id
        self.name = name
        self._boards = dict()

        if isinstance(boards, list):
            for bd in boards:
                self.add_board(bd)
        elif isinstance(boards, dict):
            for name, bd in boards.items():
                self.add_board(bd, name)
        else:
            ValueError("require list or dict.")
        for name, value in kwargs.items():
            setattr(self, name, value)


    def add_board(self, board, alias=None):
        """Add a board to this union

        Arguments:
            board {Board object} -- mcutk.board.Board object

        Keyword Arguments:
            alias {string} -- alias name in union (default: {None}),
                if is None, alias default will use Board.devicename

        Raises:
            ValueError -- raise When alias already in union.
        """

        assert isinstance(board, Board)

        if not alias:
            alias = board.devicename

        if alias in self._boards:
            raise ValueError("duplicate alias name in boardunion")

        self._boards[alias] = board
        setattr(self, alias, board)
        board.register_resource(self, "board_union")


    def remove_board(self, name):
        """remove a board

        Arguments:
            name {string} -- alias in union

        Raises:
            ValueError -- raise when name not in unio.
        """
        try:
            self._boards[name].remove_resource(self)
            del self._boards[name]
            delattr(self, name)
        except KeyError:
            raise ValueError("name is not in this union")




    def check_status(self):
        """Check boards status.
        """
        for bd in self._boards:
            status = bd.check_serial()
            print(bd.name + " " + status)
            if status != "pass":
                return status
        else:
            return "pass"




from __future__ import print_function
import os
import logging
import time
import click
from collections import defaultdict
from mcutk.apps import appfactory
from mcutk.exceptions import ProjectNotFound, ProjectParserError
from globster import Globster


TOOLCHAINS = [
    'iar',
    'mdk',
    'mcux',
    'armgcc',
    'lpcx'
]


EXCLUDE_DIR_NAME = [
    'log/',
    'debug/',
    'obj/',
    'release/',
    '.debug/',
    '.release/',
    'RTE/',
    'settings/'
    '.git/',
    '__pycache__/',
    'flexspi_nor_debug/',
    'flexspi_nor_release/'
]


def find_projects(root_dir, recursive=True, include_tools=None, exclude_tools=None):
    """Find projects in specific directory.

    Arguments:
        root_dir {string} -- root directory
        recursive {bool} -- recursive mode
        include_tools {list} -- only include specifices tools
        exclude_tools {list} -- exlucde specifices tools
    Returns:
        {dict} -- key: toolchain name, value: a list of Project objects.

    Example:
        >> ps = find_projects("C:/code/mcu-sdk-2.0", True)
        >> ps
        {
            'iar': [<Project Object at 0x1123>, <Project Object at 0x1124>],
            'mdk': [<Project Object at 0x1123>, <Project Object at 0x1124>],
            ...
        }

    """
    ide_classes = [appfactory(toolname) for toolname in TOOLCHAINS]
    projects = defaultdict(list)

    g = Globster(EXCLUDE_DIR_NAME)

    def get_project(path):
        prj = None
        for cls in ide_classes:
            try:
                prj = cls.Project.frompath(path)
                break
            except ProjectParserError as e:
                logging.warning(str(e))
            except ProjectNotFound:
                pass

        return prj

    print('Process scanning')
    s_time = time.time()

    project = get_project(root_dir)
    if project:
        projects[project.idename].append(project)

    if recursive:
        for root, folders, _ in os.walk(root_dir):
            for folder in folders:
                if g.match(folder):
                    continue

                path = os.path.join(root, folder)
                project = get_project(path)
                if (project and project.idename != "mcux") or (hasattr(project, "is_enabled") and (project.is_enabled)):
                    projects[project.idename].append(project)

    if projects:
        if include_tools:
            projects = {k: v for k, v in projects.items()
                        if k in include_tools}

        elif exclude_tools:
            for toolname in exclude_tools:
                if toolname in projects:
                    projects.pop(toolname)

    e_time = time.time()

    count = 0
    for toolname, prjs in projects.items():
        length = len(prjs)
        count += length

    click.echo("Found projects total {0}, cover {1} toolchains. Used {2:.2f}(s)".format(
        count, len(projects), e_time-s_time))

    for toolname, prjs in projects.items():
        length = len(prjs)
        click.echo(" + {0:<10}  {1}".format(toolname, length))

    return projects, count



import sys
import os
import click
import logging
from mcutk import __version__





class ComplexCLI(click.MultiCommand):

    COMMANDS = [
        'build',
        'scan',
        'config',
        # 'gdbserver',
        'flash'
    ]

    def list_commands(self, ctx):
        return self.COMMANDS


    def get_command(self, ctx, name):
        try:
            if sys.version_info[0] == 2:
                name = name.encode('ascii', 'replace')
            mod = __import__('mcutk.commands.' + name,
                             None, None, ['cli'])
        except ImportError:
            return
        return mod.cli



@click.command(cls=ComplexCLI, invoke_without_command=True, help="mcutk command line tool")
@click.option('--version', is_flag=True, help="show mcutk version")
def main(version=False, debug=False):

    if version:
        click.echo(__version__)



if __name__ == '__main__':
    main()



from __future__ import print_function
import os
import glob
import logging
import shlex
import subprocess

from mcutk.util import run_command
from mcutk.exceptions import ProcessTimeout, InstallError
from mcutk.apps.idebase import Result, DEFAULT_MCUTK_WORKSPACE



def build(func):
    """This function is Decorator and it a will start a new process to exectue the commands which
    comes from APP.build_project. When the process finished, it will return the build results.

    Default timeout: 1800 seconds

    Returns:
        mcutk.apps.BuildResult object.

    Notice:
        For new IDE support, you'd better to add this decorator on the function APP.build_project.
    """
    def wraper(*args, **kwargs):
        output = None
        timeout = kwargs.pop('timeout') if 'timeout' in kwargs else 1800
        _toolchain, _project, _target, _logile = args

        if not kwargs.get('workspace'):
            kwargs['workspace'] = DEFAULT_MCUTK_WORKSPACE + "/" + _toolchain.name

        cmdline = func(*args, **kwargs)


        # shell=True: this can resolve windows backslash
        returncode = run_command(cmdline, shell=True, stdout=False, timeout=timeout, need_raise=True)[0]
        br = _toolchain.parse_build_result(returncode, _logile)

        output = _project.targetsinfo.get(_target)
        br.set_output(output)

        # errors
        if br.result not in (Result.Pass, Result.Warnings):
            return br

        workspace = kwargs.get('workspace')

        # For eclipse projects, output is located in workspace
        if _toolchain.name in ("mcux") and workspace:
            output_abs = os.path.join(workspace, output)
        else:
            # Assume output file is located project root
            output_abs = os.path.join(_project.prjdir, output)

        # File exists, return diectly
        if os.path.isfile(output_abs):
            br.set_output(output_abs)
            return br

        if not os.path.exists(output_abs):
            logging.warning("output is not exists: [%s]", output_abs)
            return br


        # Find files in output diectory
        valid_extension= ('.axf', '.elf', '.out', '.hex', '.bin', '.lib', '.a')
        for ext in valid_extension:
            try:
                file_path = glob.glob(output_abs + "/*"+ ext)[0]
                br.set_output(file_path)
                break
            except IndexError:
                pass
        else:
            logging.warning("unable to find output: [%s]", output_abs)

        return br

    return wraper








def appinstall(func):
    """Decorator for app installation
        1. Pre change file mode: 0777.
        2. Post check the path
    """
    def wraper(*args, **kwargs):
        installer = args[0]
        logging.info("Prepare to install: %s"%installer)
        os.chmod(installer, 0o777)
        path, version = func(*args, **kwargs)
        if not path or not os.path.exists(path):
            logging.error("Return path is not exists: %s", str(path))
            raise InstallError("Failed to install app!")
        return path, version

    return wraper






import os
import sys
import re
import platform
import subprocess
import logging
import threading
import time
import tempfile
import json



from mcutk.debugger.base import DebuggerBase
from mcutk.util import run_command


PY = sys.version_info[0]

DEFAULT_TIMEOUT = 600000


def find_hid_devices():
    """Use pywinusb as the backend to discovery HID devices, windows only."""

    from pywinusb import hid as hid

    devices = list()
    hid_devices = hid.find_all_hid_devices()

    for dev in hid_devices:
        # filter vid is: 0x15a2
        if hex(dev.vendor_id) == "0x15a2":
            print(dev.device_path)
            print('---------------------------')
            devices.append(dev)

    return devices





class Response(object):
    """Represent a blhost response."""

    @classmethod
    def parse(cls, raw):
        response = cls()
        response.output = raw

        match = re.search(r"{[\s\S]+}", raw)
        if match:
            raw_data = json.loads(match.group(0))
            response.content = raw_data.get('response') #raw.replace(match.group(0), '').strip()
            response.command = raw_data.get('command')
            response.status_value = raw_data.get('status').get('value')
            response.description = raw_data.get('status').get('description')
        else:
            response.content = raw
            if "Response status = 0 (0x0) Success" in raw:
                response.status_value = 0
            else:
                response.status_value = -1
        return response


    def __init__(self, **kwargs):
        self.status_value = kwargs.get("status_value")
        self.description = kwargs.get("status_description")
        self.command = kwargs.get("command")
        self.output = kwargs.get("output")
        self.value = kwargs.get("value")

    def __str__(self):
        return "<CMD: {}, {}>".format(self.command, self.description)


    @property
    def status(self):
        return self.status_value




class Blhost(DebuggerBase):
    """Blhost is application running on host PC and used to issue commands to MCU bootloader.
        Guides online: https://www.nxp.com/docs/en/user-guide/MCUBLHOSTUG.pdf.

    This class wrapped blhost tool, it always set "--json" to get formatted JSON string,
    and when command is run successful, a Response object will be returned.

    Example, initial blhost debugger
        >>> bl = Blhost('C:/user/blhost.exe')
        >>> bl.set_connection("-p COM3,56700")                  # set connection
        >>> bl.set_connection("-u")                             # set connection
        >>> resp = bl.run_command("get-property 1")             # send command get-property 1
        >>> resp = bl.run_command("get-property 1", json=False) # send command get-property 1, none json output
        >>> bl.reset()                                          # reset chip
        >>> bl.erase()                                          # erase flash


    Response object:
        >>> resp = bl.run_command("get-property 2")
        >>> print resp.status_code
        0

        >>> print resp.value
        [ 23 ]

        >>> print resp.command
        get-property 2

        >>> print resp.status
        True

        >>> print resp.output
        {
        "command" : "get-property",
        "response" : [ 23 ],
        "status" : {
            "description" : "0 (0x0) Success.",
            "value" : 0
            }
        }


    Run commands without constructor
        >>> resp = Blhost.run("blhost -p COM3 -- get-property 1")
        >>> print resp.status
        True
        >>> print resp.output
        Inject command 'get-property'
        Response status = 0 (0x0) Success.
        Response word 1 = 1258424064 (0x4b020700)
        Current Version = K2.7.0

    """

    @staticmethod
    def run(cmd, **kwargs):
        """Run blhost command directly"""

        if '-t' in cmd:
            timeout = 1200
        else:
            timeout = 30

        rc, output = run_command(cmd, stdout=True, timeout=timeout)
 
        # print Response.parse(output)       
        if rc != 0:
            raise RuntimeError("blhost command error: %s"%rc)

        response = Response.parse(output)

        logging.info(output)

        return response




    def __init__(self, *args, **kwargs):
        """Blhost debugger constructor."""

        super(Blhost, self).__init__("blhost", *args, **kwargs)
        if self.path:
            self._blhost_exe = self.path
        else:
            self._blhost_exe = 'blhost'
        self._connect_opt = "-u"



    @property
    def is_ready(self):
        return os.path.exists(self._blhost_exe)



    def set_connection(self, type=None, value=None):
        """Set blhost connection. Default automaticlly select usb to connect.

        Manual to set connection options by value:
            - port: -p/--port <name>[,<speed>]
            - usb: -u/--usb [[[<vid>,]<pid>] | [<path>]]

            Example:
            - set connection with port:
                blhost.set_connection(value="--port COM3,57600")
            - set connection with usb hid:
                blhost.set_connection(value="--usb 0x15a2,0x0073")

        Arguments:
            type {str} -- usb or port.
            value {str} -- port,baudrate | vid,pid
        """

        if value:
            self._connect_opt = value

        elif type == 'port':
            if self._board.main_serial and self._board.main_serial.port:
                self._connect_opt = "--port {}".format(self._board.main_serial.port)
            else:
                logging.warning("port is not configured")

        elif type == 'usb':
            devices = find_hid_devices()
            if not devices:
                logging.warning("Not found hid devices with VID: %s!", "0x15a2")

            elif len(devices) > 1:
                logging.warning("Multiple devices found!")
                self._connect_opt = "--usb \"{}\"".format(devices[0].device_path)

        if not self._connect_opt:
            logging.warning("connect options is not set, forget to set board?")



    def _build_command(self, cmd, json, memid, timeout):
        assert self._connect_opt

        command = [
            "{exe}",
            "{connect_opt}",
            "-t {0}".format(timeout) if timeout else '',
            "--json" if json else "",
            "--",
            "{cmd}",
            "{memid}".format(memid=memid) if memid != None else ""
        ]

        return " ".join(command).format(exe=self._blhost_exe, connect_opt=self._connect_opt, cmd=cmd)

    def eval_timeout(self, byte_count):
        '''
        Evalute the timeout for fill-memory/ write-memory/ read-memory
        '''
        try:
            byte_count = eval(byte_count)
        except TypeError:
            pass

        if byte_count in range(0, 0x172000):
            timeout = DEFAULT_TIMEOUT / 2
        elif byte_count in range(0x172000, 0x2E4000):
            timeout = DEFAULT_TIMEOUT
        else:
            timeout = DEFAULT_TIMEOUT * 2
        return timeout

    def run_command(self, cmd, json=True, memid=None, timeout=None):
        """Run a single blhost command, and return a blhost response object.

        Arguments:
            cmd -- {str} blhost command
            json -- {boolean} json formmatted output
            memid -- {str/int} memory ID

        Returns:
            Response object

        blhost-commands:
            reset
            get-property
            set-property
            flash-erase-region
            flash-erase-all
            flash-erase-all-unsecure
            read-memory
            write-memory
            fill-memory
            receive-sb-file
            execute
            call
            flash-security-disable
            flash-program-once
            flash-read-once
            flash-read-resource
            configure-memory
            reliable-update
            key-provisioning
            flash-image
            list-memory
            efuse-program-once
            efuse-read-once
            program-aeskey
            generate-key-blob
        """

        blhost_cmd = self._build_command(cmd, json, memid, timeout)
        logging.info(blhost_cmd)
        return Blhost.run(blhost_cmd)



    def run_commands(self, commands, **kwargs):
        """Run a list of commands. RuntimeError will be raised when one of command failed."""

        for cmd in commands:
            resp = self.run_command(cmd, **kwargs)
            if not resp.status:
                raise RuntimeError("command run failed, %s"%str(resp))



    def test_conn(self):
        """Test debugger connection."""
        resp = self.run_command("get-property 1")
        return resp



    def erase(self, memid=None):
        """rase all flash according to [memory_id],
            excluding protected regions.

        blhot: flash-erase-all
        """
        self.run_command('flash-erase-all', memid=memid)



    def reset(self):
        """Use blhost to reset chip."""

        return self.run_command("reset")



    def read_mem(self, addr, length, file=None, memid=None):
        """Read memory from a given address.
        If file is set, read memory will to file.

        blhost: read-memory <addr> <byte_count> [<file>] [memory_id]
        """
        timeout = self.eval_timeout(length)
        cmd = "read-memory {addr} {length}".format(addr=addr, length=length)

        if file:
            cmd += " %s "%file

        return self.run_command(cmd, memid=memid, timeout=timeout)


    def write_mem(self, addr, value=None, file=None, memid=None):
        """Write memory.

        blhost: write-memory <addr> [<file>[,byte_count]| {{<hex-data>}}] [memory_id]
        """
        
        cmd = "write-memory {addr}".format(addr=addr)

        if value:
            cmd += " %s "%value
            timeout = None

        elif file:
            cmd += " %s "%file
            file_size = os.path.getsize(file)
            timeout = self.eval_timeout(file_size)

        return self.run_command(cmd, memid=memid, timeout=timeout)

    def fill_memory(self, addr, length, pattern ,pattern_mode = "word"):
        """Fill memory with pattern; size is word (default), short or byte
        blhost: <addr> <byte_count> <pattern> [word | short | byte]
        """
        timeout = self.eval_timeout(length)
        cmd = "fill-memory {addr} {length} {pattern} {pattern_mode}".format(addr=addr, length=length, pattern=pattern, pattern_mode=pattern_mode)

        return self.run_command(cmd, timeout=timeout)


    def flash(self, filepath, erase='erase', memid=None):
        """Flash a formated image <file> to memory with ID <memory_id>.
        Supported file types: SRecord(.srec and .s19) and HEX (.hex). Flash is erased
        before writing if [erase]=erase. The erase unit size depends on the target and
         the minimum erase unit size is 1K.

        Arguments:
            filepath {str} -- image file path
            erase {int} -- erase flash or not before flashing. 
            memid {int} --

        """

        cmd = "flash-image {file}".format(file=filepath)

        if erase:
            cmd += ' %s'%erase
        else:
            pass

        return self.run_command(cmd, memid=memid)

    def get_property(self, tag, memid_index = None, json=True):
        """
        Return bootloader specific property.
        blhost: get-property <tag> [<memid>]
        """

        cmd = "get-property {tag}".format(tag=tag)

        return self.run_command(cmd, memid=memid_index, json=json)

    def set_property(self, tag, value):
        """
        Set the specified property.file
        blhost: set-priperty <tag> <value>
        """

        cmd = "set-property {tag} {value}".format(tag=tag, value=value)
        return self.run_command(cmd)

    def flash_erase_region(self, addr, byte_count, memid=None):
        """
        Erase a region of flash according to [memory_id]
        blhost: flash-erase-region <addr> <byte_count> [memory_id]
        """
        timeout = self.eval_timeout(byte_count)

        cmd = "flash-erase-region {addr} {byte_count}".format(addr=addr, byte_count=byte_count)
        return self.run_command(cmd, memid=memid, timeout=timeout)

    def flash_erase_all_unsecure(self):
        """
        Erase all internal flash, including protected regions
        blhost: flash-erase-all-unsecure
        """
        return self.run_command("flash-erase-all-unsecure", timeout=DEFAULT_TIMEOUT)

    def flash_erase_all(self, memid=0):
        '''
        Erase all flash according to [memid], excluding protected regions
        blhost: flash-erase-all <memid>

        '''
        return self.run_command('flash-erase-all', memid=memid, timeout=DEFAULT_TIMEOUT)

    def receive_sb_file(self, file):
        """Receive SB file
        blhost: receive-sb-file <file>
        """

        cmd = "receive-sb-file {file}".format(file=file)
        return self.run_command(cmd, timeout=DEFAULT_TIMEOUT)

    def execute(self, addr, arg, stackpointer):
        """
        Execute at address with arg and stack pointer
        blhost: execute <addr> <arg> <stackpointer>
        """
        cmd = "execute {addr} {arg} {stackpointer}".format(addr=addr, arg=arg, stackpointer=stackpointer)
        return self.run_command(cmd)

    def call(self, addr, arg):
        """
        Call address with arg
        blhost: call <addr> <arg>
        """

        cmd = "call {addr} {arg}".format(addr=addr, arg=arg)
        return self.run_command(cmd)

    def flash_program_once(self, index, byte_count, value, mode= "LSB"):
        """
        Program Flash Program Once Field
        blhost: flash-program-once <index> <byte_count> <data> [LSB | MSB]
        """

        cmd = "flash-program-once {index} {byte_count} {value} {mode}".format(index=index, byte_count=byte_count, value=value, mode=mode)
        return self.run_command(cmd)

    def flash_read_once(self, index, byte_count):
        """
        Read Flash Program Once Field
        blhost: flash-read-once <index> <byte_count>
        """

        cmd = "flash-read-once {index} {byte_count}".format(index=index, byte_count=byte_count)
        return self.run_command(cmd)

    def flash_read_resource(self, addr, byte_count, option, file = None):
        """
        Read Resource from special-purpose, non-volatile memory and write to file, or stdout if no file specified
        """

        cmd = "flash-read-resource {addr} {byte_count} {option}".format(addr=addr, byte_count=byte_count, option=option)

        if file:
            cmd += " %s "%file
        else:
            pass

        return self.run_command(cmd)

    def configure_memory(self, memid, internal_addr):
        """
        Apply configuration block at internal memory address <internal_addr> to memory with ID <memory_id>
        blhost: configure-memory <memory_id> <internal_addr>
        """
        cmd = "configure-memory {memid} {internal_addr}".format(memid=memid, internal_addr=internal_addr)

        return self.run_command(cmd)

    def generate_key_blob(self, dek_file, blob_file):
        """
        Generate the Blob for given Dek Key
        <dek_file> - input, a binary Dek Key (128 Bits) generated by CST tool.
        <blob_file> - output, a generated blob (72 Bytes) in binary format.
        blhost: generate-key-blob <dek_file> <blob_file>
        """

        cmd = "generate-key-blob {dek_file} {blob_file}".format(dek_file=dek_file, blob_file=blob_file)

        return self.run_command(cmd)

    def reliable_update(self, addr):
        """
        Copy backup app from address to main app region or swap flash using indicator address
        blhost: reliable-update <addr>
        """

        cmd = "reliable-update {addr}".format(addr=addr)

        return self.run_command(cmd)

    def key_provisioninig_enroll(self):
        """
        Key provisioning enroll. No argument for this operation.
        blhost: key-provisioning enroll
        """

        cmd = "key-provisioning enroll"

        return self.run_command(cmd)

    def key_provisioning_set_user_key(self, typeid, file, size=None):
        """
        Send the user key specified by <type> to bootloader. 
        <file> is the binary file containing user key plaintext. If <size> is not specified, the entire <file> will be sent. 
        Otherwise, only send the first <size> bytes.
        blhost: key-provisioning set_user_key <type> <file> [,<size>]
        """

        cmd = "key-provisioning set_user_key {type} {file}".format(type=typeid, file=file)

        if size:
            cmd += ",%s "%size
        else:
            pass

        return self.run_command(cmd)

    def key_provisioning_set_key(self, typeid, size):
        """
        Generate <size> bytes of the key specified by <type>.
        blhost: key-provisioning set_key <type> <size>
        """

        cmd = "key-provisioning set_key {type} {size}".format(type=typeid, size=size)

        return self.run_command(cmd)

    def key_provisioning_write_key_nonvolatile(self, memid):
        """
        Write the key to a nonvolatile memory to bootloader.
        blhost: key-provisioning write_key_nonvolatile <memid>
        """

        cmd = "key-provisioning write_key_nonvolatile {memid}".format(memid=memid)

        return self.run_command(cmd)

    def key_provisioning_read_key_nonvolatile(self, memid):
        """
        Load the key from a nonvolatile memory to bootloader.
        blhost: key-provisioning read_key_nonvolatile <memid>
        """

        cmd = "key-provisioning read_key_nonvolatile {memid}".format(memid=memid)

        return self.run_command(cmd)

    def key_provisioning_write_key_store(self, file, size=None):
        """
        Send the key store to bootloader. 
        <file>is the binary file containing key store. 
        If <size> is not specified, the entire <file> will be sent. 
        Otherwise, only send the first <size> bytes.

        blhost: key-provisioning write_key_store <file>[,<size>]
        """

        cmd = "key-provisioning write_key_store {file}".format(file=file)

        if size:
            cmd += ",%s "%size
        else:
            pass

        return self.run_command(cmd)

    def key_provisioning_read_key_store(self, file):
        """
        Read the key store from bootloader to host(PC). <file> is the binary file to store the key store.
        blhost: key-provisioning read_key_store <file>
        """

        cmd = "key-provisioning read_key_store {file}".format(file=file)

        return self.run_command(cmd)

    def list_memory(self):
        """
        List all on-chip Flash and RAM regions, and off-chip memories, supported by current device.
        Only the configured off-chip memory will be list.
        blhost: list-memory
        """
        return self.run_command("list-memory", json=False)

    def efuse_program_once(self, addr, data, iflock=None):
        """
        Program one word of OCOTP Filed.
        <addr> is ADDR of OTP word, not the shadowed memory address.
        <data> is hex digits without prefix '0x'
        """
        cmd = "efuse-program-once {addr} {data}".format(addr=addr, data=data)

        if iflock:
            cmd += " %s "%iflock
        else:
            pass

        return self.run_command(cmd)

    def efuse_read_once(self, addr):
        """
        Read one word of OCOTP Field.
        <addr> is ADDR of OTP word, not the shadowed memory address.
        blhost: efuse-read-once <addr>
        """

        cmd = "efuse-read-once {addr}".format(addr=addr)

        return self.run_command(cmd)



from __future__ import absolute_import

import os
import logging
from distutils.version import LooseVersion


try:
    # pyocd version >= 0.13
    from pyocd.target import TARGET
    from pyocd.core.helpers import ConnectHelper
    from pyocd import __version__ as pyocd_version
except ImportError as e:
    # old pyocd version
    try:
        from pyOCD.target import TARGET
        from pyOCD.board import MbedBoard
        from pyOCD import __version__ as pyocd_version

    except ImportError:
        pyocd_version = ''
        print('Fatal Error: pyocd is missing, please install it at first!')

from mcutk.debugger.base import DebuggerBase
from mcutk.util import run_command


class PYOCD(DebuggerBase):
    """
    Wrap pyOCD to a standard debugger for pymcutk.

    pyOCD is an open source Python package for programming and debugging Arm Cortex-M microcontrollers
    using multiple supported types of USB debug probes. It is fully cross-platform, with support for Linux,
    macOS, and Windows. For more visit https://github.com/mbedmicro/pyOCD.
    """

    def __init__(self, **kwargs):
        super(PYOCD, self).__init__("pyocd", ".", **kwargs)
        self.version = pyocd_version
        self._target_option = ""

        if self.version and LooseVersion(self.version) >= LooseVersion("0.13.0"):
            self._gdbserver = "pyocd gdbserver"
            self._flashtool = "pyocd flash"
            self._pyocdtool = "pyocd-tool"
        else:
            # windows pyocd tools are regenerated by ourselves
            # use these commands directly.
            if os.name == "nt":
                self._gdbserver = "pyocd-gdbserver"
                self._flashtool = "pyocd-flashtool"
                self._pyocdtool = "pyocd-tool"
            else:
                self._gdbserver = "python -m pyOCD.tools.gdb_server"
                self._flashtool = "python -m pyOCD.tools.flash_tool"
                self._pyocdtool = "python -m pyOCD.tools.pyocd"


    @property
    def is_ready(self):
        return self.version not in (None, '')



    def set_board(self, board):
        self._board = board
        device_name = board.devicename
        if device_name.endswith('pack') and os.path.exists(device_name):
            self._target_option = " --pack %s"%device_name
        else:
            self._target_option = self._overried_target(device_name)


    def _overried_target(self, devicename):
        """Overried target if device name is include in pyOCD.target.TARGET.
        This is very useful if you want to use other device target for debugging.
        """
        if self.is_ready and devicename in TARGET:
            return " -t %s"%devicename

        return ""


    def list_connected_devices(self):
        """List connected CMSIS-DAP devices."""

        probes = ConnectHelper.get_all_connected_probes(blocking=False)
        devices = list()
        for probe in probes:
            device = {
                'usbid': probe.unique_id,
                'name': probe.description,
                'type': 'pyocd'
            }
            devices.append(device)

        return devices



    def test_conn(self):
        """Test debugger connection."""

        if self._board is None:
            raise ValueError("board is not set")

        mbed = None
        msg = "NoError"
        try:
            mbed = MbedBoard.chooseBoard(self._board.usbid)
        except Exception as e:
            msg = "ConnectError: %s"%(str(e))
        finally:
            if mbed:
                mbed.uninit()

        return msg



    def erase(self):
        """Erase flash by: pyocd-tool erase -b <id>
        """
        logging.info("pyocd erase flash")
        if LooseVersion(self.version) >= LooseVersion("0.13.0"):
            command = "pyocd erase --mass-erase -u {0} {1}".format(self._board.usbid, self._target_option)
        else:
            command = "{0} --mass-erase -d error -b {1} {2}".format(self._flashtool, self._board.usbid, self._target_option)
        return run_command(command)[0]



    def reset(self):
        """Reset board by: pyocd-tool reset -b <id>"""

        logging.info("pyocd reset board")
        command = "{0} reset -d error -b {1} {2}".format(self._pyocdtool, self._board.usbid, self._target_option)
        return run_command(command)[0]



    def api_reset(self, usbid):
        """Reset board via pyOCD API."""

        logging.info("resetting board")
        try:
            if self.version and LooseVersion(self.version) >= LooseVersion("0.13.0"):
                with ConnectHelper.session_with_chosen_probe(unique_id=usbid) as session:
                    target = session.board.target
                    target.reset()
            else:
                with MbedBoard.chooseBoard(board_id=usbid) as mboard:
                    mboard.target.reset()

            return True

        except Exception as e:
            logging.exception("resetting board exception")
            return False



    def unlock(self):
        """Unlock board by: pyocd-tool.exe unlock -b <id>"""

        logging.info("unlock board")
        command = "{0} unlock -b {1} {2}".format(self._pyocdtool, self._board.usbid, self._target_option)
        return run_command(command)[0]



    def get_gdbserver(self, **kwargs):
        """Return gdb server command. If configured board.devicename is a target
        name in pyocd.target.TARGET. This will overried the target.
        """

        board = kwargs.get('board')
        port = kwargs.get('port')
        usbid = kwargs.get('usbid')
        devicename = kwargs.get('devicename')

        if board is None:
            board = self._board

        if board:
            port = board.gdbport
            usbid = board.usbid
            devicename = board.devicename

        if self.version and LooseVersion(self.version) >= LooseVersion("0.13.0"):
            command = "pyocd gdbserver --trust-crc"
            if usbid:
                command += " --uid %s"%usbid
            if port:
                command += " --port %s"%port

        else:
            command = self._gdbserver
            if usbid:
                command += " -b %s"%usbid
            if port:
                command += " -p %s"%port

        if devicename:
            command += " " + self._target_option

        return command



    def flash(self, filepath, erase=False, addr=0):
        """Flash chip with filepath. (bin, hex).

        Arguments:
            erase: erase chip yes or not.
            addr: being the integer starting address for the bin file.
        """
        filepath = filepath.replace("\\", "/")

        if LooseVersion(self.version) < LooseVersion("0.13.0"):
            command = "{0} -b {1} -a {2} {3}".format(
                self._flashtool,
                self._board.usbid,
                self._board.start_address,
                filepath)
        else:
            # --trust-crc: use only the CRC of each page to determine if it
            # already has the same data
            command = "{0} -u {1} -a {2} {3} --trust-crc".format(
                self._flashtool,
                self._board.usbid,
                self._board.start_address,
                filepath)

        command += self._target_option

        # call registerd callback function
        self._call_registered_callback("before-load")
        return run_command(command, timeout=300)





    def gdb_init_template(self):
        """Default PyOCD gdb commands temaplte.

        >>>
            set tcp connect-timeout 10
            set remotetimeout 10
            target remote localhost: {gdbport}
            load
            q
        """
        commands = '''set tcp connect-timeout 10
set remotetimeout 100
target remote localhost: {gdbport}
load
monitor reset
q
'''
        return commands



    def __str__(self):
        return "pyocd-%s" % self.version




    @staticmethod
    def get_latest():
        de = PYOCD()
        return de if de.is_ready else None





import os
from mcutk.debugger import jlink
from mcutk.debugger import pyocd
from mcutk.debugger import redlink
from mcutk.debugger import ide
from mcutk.debugger import blhost

__all__ = ["getdebugger", 'jlink', 'pyocd', 'redlink']



def getdebugger(type, *args, **kwargs):
    """Return debugger instance."""
    supported = {
        "jlink": jlink.JLINK,
        "pyocd": pyocd.PYOCD,
        "redlink": redlink.RedLink,
        'ide': ide.IDE,
        'blhost': blhost.Blhost
    }
    try:
        return supported[type](*args, **kwargs)
    except KeyError:
        raise ValueError("not supported debugger: %s"%type)




from __future__ import absolute_import
from functools import partial
import io
import logging

from serial import Serial as PY_SERIAL
from serial.threaded import Protocol, ReaderThread
from mcutk.pserial.serialspawn import SerialSpawn


class DataHandler(Protocol):
    """Data handler for ReaderThread."""

    def connection_made(self, transport):
        super(DataHandler, self).connection_made(transport)
        self._serial = transport.serial

    def data_received(self, data):
        if data:
            self._serial._data.append(data)





class Serial(PY_SERIAL):
    """This class is inherited from pyserial::serial.Serial class.
    It extended the Serial class to support data reading in a background thread.

    The attribute serial.reader is the instance of reading thread.


    spawn = serila.SerialSpawn()
    when you close.spawn
    """

    def __init__(self, *args, **kwargs):
        if not kwargs.get('timeout', None):
            kwargs['timeout'] = 1
        super(Serial, self).__init__(*args, **kwargs)
        self._data = list()
        self.reader = None
        # enable serialspawn, default logfile_read is memory
        self.Spawn = partial(SerialSpawn, self, logfile_read=io.BytesIO())
        self.SerialSpawn = self.Spawn


    def write(self, data, log=True):
        """write data"""
        if log:
            logging.info("%s write: %s", self.port, repr(data))
        super(Serial, self).write(data)


    def start_reader(self):
        """Start the reader thread, and return the data handler when the reader is running.
        If the port is not open, it will open it at first.
        """
        if not self.is_open:
            self.open()

        if self.reader_isalive:
            raise RuntimeError('failed to start reader, reader thread is already running.')

        self.reader = ReaderThread(self, DataHandler)
        data_handler = self.reader.__enter__()
        logging.info('%s reading thread is running!', self.port)
        return data_handler


    def stop_reader(self):
        """Stop the reader thread."""
        if self.reader_isalive:
            self.reader.stop()
            logging.info('%s reading thread is stopped!', self.port)


    def clear_reader_buffer(self):
        """Clear the data buffer for the reader thread.
        """
        self._data = list()


    @property
    def reader_isalive(self):
        """Return a boolean value to identify the reader is alive or not.
        """
        return self.reader and self.reader.alive



    @property
    def data(self):
        """Return all of data in the internal data buffer."""
        return "".join(self._data)


    def append_data(self, data):
        """Append data to internal buffer."""
        self._data.append(data)



    def close(self):
        """Close the serial port.
        """
        if self.reader and self.reader.alive:
            self.stop_reader()
        super(Serial, self).close()




"""
Repo operation example
"""
import os
import sys

sys.path.append("./../")

from mcutk.repo import Repo

# create instance from path
repo = Repo.frompath("C:/mcu-sdk-2.0")
print repo.repo_url
print repo.is_ready
print repo.get_branch_name()
print repo.get_latest_commit_message()
print repo.get_submodules()

repo.checkout_branch("master")
repo.AutoSync("master")


# create repo instance by url
repo = Repo("ssh://git@bitbucket.sw.nxp.com/mcucore/mcu-sdk-2.0.git")
print repo.repo_url
print repo.is_ready
print repo.get_branch_name()
print repo.get_latest_commit_message()
print repo.get_submodules()

repo.Clone(".", "master")
repo.AutoSync(".", "develop")




import os
import re
import glob

from mcutk.apps.projectbase import ProjectBase
from mcutk.exceptions import ProjectNotFound

class Project(ProjectBase):
    """
    ARMGCC project object

    This class could parser the settings in CMakeLists.txt & build_all.sh.
    Parameters:
        prjpath: path of CMakeLists.txt

    """

    PROJECT_EXTENSION = 'CMakeLists.txt'

    @classmethod
    def frompath(cls, path):
        """Return a project instance from a given file path or directory.

        If path is a directory, it will search the project file and return an instance.
        Else this will raise mcutk.apps.exceptions.ProjectNotFound.
        """

        if os.path.isfile(path):
            return cls(path)

        if glob.glob(path + "/CMakeLists.txt") and glob.glob(path + "/build_all.*"):
            return cls(path + "/CMakeLists.txt")

        raise ProjectNotFound("Not found armgcc project in path: %s"%path)



    def __init__(self, prjpath, *args, **kwargs):
        super(Project, self).__init__(prjpath, *args, **kwargs)
        self._appname = None
        self._conf = self._get_all_configuration()
        self._targets = self._conf.keys()
        self.armgcc_cmake = self.__get_armgcc_cmake()



    @property
    def name(self):
        """Return the application name

        Returns:
            string --- app name
        """
        return self._appname




    def _get_all_configuration(self):
        """Parse settings from CMakeLists.txt.

        Returns:
            dict -- targets configuration
        """
        targets = dict()

        with open(self.prjpath, 'r') as fh:
            content = fh.read()

        # extract output name
        output_keywords = [
            r'add_library\(.+(\.)?\w+',
            r'add_executable\(.+(\.)?\w+',
            r'set_target_properties\(\w+(\.)?\w+',
            r'TARGET_LINK_LIBRARIES'
        ]

        excutable = None
        for kw in output_keywords:
            s = re.compile(kw).search(content)
            if s != None:
                excutable = s.group(0).split('(')[1].strip()
                break
        else:
            raise ValueError("Unable to detect output definition in CMakeLists.txt. [%s]"%self.prjpath)
        self._appname = excutable.split('.')[0]

        # extract build types
        for m in re.findall("CMAKE_C_FLAGS_\w+ ", content):
            tname = m.replace('CMAKE_C_FLAGS_', '').lower().strip()
            if tname not in targets:
                targets[tname] = "{}/{}".format(tname, excutable)

        return targets




    def __get_armgcc_cmake(self):
        try:
            script_file = glob.glob(os.path.dirname(self.prjpath) + "/build_all.*")[0]
        except Exception:
            raise IOError("Cannot find build_all.* file,armgcc can not read info from this file!")

        with open(script_file, "r") as fh:
            filecontent = fh.readlines()

        armgcc_cmake = ''
        for line in filecontent:
            if "-DCMAKE_TOOLCHAIN_FILE=" in line:
                armgcc_cmake = line.split("-DCMAKE_TOOLCHAIN_FILE=")[1].split(" ")[0]
                break

        return armgcc_cmake





import os
import sys
import click
import glob
import logging
from os.path import join
from collections import defaultdict, Counter
import json
import yaml

from mcutk.projects_scanner import find_projects
from mcutk.managers.conf_mgr import ConfMgr


SDK_TYPES = [
    'driver_examples',
    'demo_apps',
    'multicore_examples',
    'wireless_examples',
    'aws_examples',
    'cmsis_driver_examples',
    'trustzone_examples',
    'usb_examples',
    'wifi_qca_examples',
]


def get_app_type(path):
    path = os.path.abspath(path).replace('\\', '/')
    parts = path.split('/')



@click.command('scan', short_help='projects scanner')
@click.argument('path', required=True, type=click.Path(exists=True))
@click.option('-o', '--output', type=click.Path(exists=False), help='dump scan results to file, file format support: json or yml.')
@click.option('--dapeng', is_flag=True, default=False, help='dump for dapeng style, casfile.yml')
def cli(path, output, dapeng):
    projects, count = find_projects(path, True)
    dataset = list()

    if output:
        extension = os.path.basename(output).split(".")[-1]
        for tname, plist in projects.items():
            for project in plist:
                dataset.append(project.to_dict())

        if extension in ('yml', 'yaml'):
            with open(output, 'w') as file:
                yaml.safe_dump(dataset, file, default_flow_style=False)
        else:
            with open(output, 'w') as file:
                json.dump(dataset, file)

        # elif format == 'dapeng':
        #     for project in projects:
        #         if project.path
        # else:
        #     pass

        click.echo("output file: %s"%output)







"""
This example show how to use mcutk.board & mcutk.debugger
to download image to board.
"""
import os
import signal
import sys
import time
import subprocess

sys.path.append("./../")

from mcutk.board import getboard
from mcutk.debugger.jlink import JLINK

# get jlink instance
jlink = JLINK.get_latest()
# set gdb path
jlink.gdbpath = "C:/MinGW/bin/gdb.exe"
# get a mcutk.board object
board = getboard("lpcxpresso54018", devicename="lpcxpresso54018", usbid="621000000")
# bind jlink debugger to frdmkl43z board
board.debugger = jlink
# register a callback that will be executed before loading image.
@board.debugger.register("before-load")
def callback(a=1, b=2):
    print a, b
    print "Callback is called"
    print "Callback end"

print board
print board.programming("C:\\Dpc\\FreeMV\\system\\app_data\\binaries\\frdmkl43z.out")[0]




import os
import yaml


class ConfMgr(object):

    CONFIG_FILE = os.path.join(os.path.expanduser('~'), '.mcutk')

    @classmethod
    def load(cls):
        return cls(cls.CONFIG_FILE)


    def __init__(self, path):
        self._path = path
        self.is_empty = True

        if not os.path.exists(path):
            self._data = dict()
        else:
            with open(path, 'r') as stream:
                self._data = yaml.safe_load(stream)
            self.is_empty = False

        if 'apps' not in self._data:
            self._data['apps'] = dict()

        if not isinstance(self._data['apps'], dict):
            self._data['apps'] = dict()


    def apps(self):
        return self._data['apps']


    def get_app(self, name):
        return self._data['apps'].get(name)


    def get_apps(self):
        return self._data['apps']


    def set_app(self, app):
        assert app.name
        info = {
            'path': str(app.path),
            'version': str(app.version)
        }
        self._data['apps'][app.name] = info

    def __str__(self):
        return str(self._data)


    def save(self):
        with open(self._path, 'w') as file:
            yaml.dump(self._data, file, default_flow_style=False)




import os
import sys
import click
import glob
import logging
from os.path import join
import yaml

from mcutk.apps import appfactory
from mcutk.board import Board
from mcutk.managers.conf_mgr import ConfMgr
from mcutk.managers.debug_helper import Debughelper

LEVELS = {
    'warning': logging.WARNING,
    'debug': logging.DEBUG,
    'info': logging.INFO
}



@click.command('flash', short_help='flash debug file to board')
@click.argument('path', required=True, type=click.Path(exists=True))
@click.option('-u', '--usbid', help='unique usb id')
@click.option('-a', '--base-address', default=0, help='base address to load.')
@click.option('-c', '--config-file', help='load config from file.')
def cli(path, usbid, base_address, config_file):
    # config logging
    level = LEVELS.get('info', logging.INFO)
    format = '[%(levelname)s] %(message)s'
    logging.basicConfig(level=level, format=format)

    # get arm-none-eabi-gdb
    gdb = None
    armgcc = appfactory('armgcc').get_latest()
    if armgcc and armgcc.is_ready:
        gdb = os.path.join(armgcc.path, 'bin/arm-none-eabi-gdb')

    # load from config file
    device = dict()
    if config_file:
        with open(config_file) as file:
            config = yaml.safe_load(file)
            device = config.get('board')

    if usbid:
        device['usbid'] = usbid

    # prepare debugger and device
    debugger, device = Debughelper.choose_device(device)
    if not device:
        exit(1)

    board = Board(**device)
    board.debugger = debugger
    board.debugger.gdbpath = gdb
    click.secho(str(board), fg="yellow")

    if base_address:
        board.start_address = base_address

    ret = board.programming(path)
    if ret[0] == 0:
        click.secho('Flash programming successful!', fg="green")
        exit(0)
    else:
        click.secho('Flash programming failed!', fg="red")
        exit(1)


import os
import glob
import platform

from xml.etree import ElementTree as ET

from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult, DEFAULT_MCUTK_WORKSPACE
from mcutk.apps import eclipse
from mcutk.util import run_command
from mcutk.elftool import transform_elf_basic


class APP(IDEBase):
    """Represent MCUXpressoIDE.
    """


    """
    MCUXpressoIDE exitcode Definition:
    --------------
        - 0 - no errors
        - 1 - no application has been found (i.e. -run option is wrong)
        - 2 - a hard internal error has occurred
        - 3 - the command line -run option returned with errors (e.g., validation, project creation, project build, etc...)
        - 4 - the command line -run option returned with warnings (e.g., validation, compile, link, etc...)
    """
    EXITCODE = {
        0 : 'PASS',
        1 : 'APP Error',
        2 : 'Internal Errors' ,
        3 : 'Errors',
        4 : 'Warnings'
    }


    OSLIST = ['Windows', "Linux", "Darwin"]


    def __init__(self, path, *args, **kwargs):
        super(APP, self).__init__("mcux", path, *args, **kwargs)
        self.builder = get_executable(self.path)



    @property
    def is_ready(self):
        return os.path.exists(self.builder)



    @build
    def build_project(self, project, target, logfile, **kwargs):
        """Return a string about the build command line.

        Arguments:
            project {mcux.Project} -- mcux project object
            target {str} -- target name
            logfile {str} -- log file path
            workspace {str} -- workspace directory

        Returns:
            string -- build commandline


        MCUXpressoIDE command line:

        - SDK Packages mode:

        Usage: <install path>/mcuxpressoide
            -application com.nxp.mcuxpresso.headless.application
            -consoleLog -noSplash
            -data             <path/to/your/workspace/folder/to/use>
            [-run  <command>  <path/to/your/build.properties> | -help <command>]

            Available commands:
                sdk.validate            Validate content of SDK(s)
                partsupport.install     Install part support from one or more SDK.
                toolchain.options       Show the supported options in the MCUXpressoIDE toolchains
                example.build           Create/build examples from one or more SDK or an examples XML definition file.
                list.probes             List all supported probes
                project.build           Create/build projects from one or more SDK.
                example.options         Check examples options.
                list.parts              List all installed MCUs

        - Eclipse mode
        """
        workspace = kwargs.get('workspace')

        if not os.path.exists(workspace):
            os.makedirs(workspace)

        file_dir = None

        # SDK package
        if project.is_package:
            # if workspace is default, properties file will save to system temps.
            # otherwise store the properties file to user's workspace
            if DEFAULT_MCUTK_WORKSPACE not in workspace:
                file_dir = workspace

            properties_file = project.gen_properties(target, file_dir)
            buildcmd = [
                self.builder,
                "--launcher.suppressErrors",
                "-noSplash",
                "-consoleLog",
                "-application",
                "\"com.nxp.mcuxpresso.headless.application\"",
                "-data",
                workspace,
                "-run",
                "example.build",
                properties_file
            ]

        # eclipse project
        else:
            buildcmd = eclipse.generate_build_cmdline(self.builder, workspace, project.prjdir, project.name, target)

        if logfile:
            buildcmd.append('>> "{}" 2>&1'.format(logfile))

        return " ".join(buildcmd)




    def programming(self, board, prjdir, target, file, **kwargs):
        """MCUXPressoIDE programming file to board.

        User must prepare a folder to involve the flash driver and device parts xml.
        """
        from mcutk.debugger.redlink import RedLink

        gdbpath = kwargs.get('gdbpath')
        redlink = RedLink(self.path + '/ide', version=self.version)
        redlink.gdbpath = gdbpath
        redlink.template_root = prjdir
        redlink.set_board(board)
        return redlink.flash(file)



    def elftool(self, type, in_file, out_file):
        """Call <mcuxpressoide>/ide/tools/bin/arm-none-eabi-objcopy to convert."""

        objcopy_exe = os.path.join(self.path, 'ide/tools/bin/arm-none-eabi-objcopy')
        if os.name == 'nt':
            objcopy_exe += '.exe'

        return transform_elf_basic(type, in_file, out_file, objcopy_exe)




    @staticmethod
    def default_install_directory():
        """MCUXpressoIDE default installation directory"""

        installation_table = {
            "Windows": "C:/nxp/",
            "Linux": "/usr/local/",
            "Darwin": "/Applications",
        }
        osname = platform.system()
        return installation_table[osname]



    @staticmethod
    def get_latest():
        """Search and return a latest IAR instance in system

        Returns:
            <mcux.APP object>
        """
        version, path = get_latest()
        if path:
            return APP(path, version=version)
        else:
            return None



    @staticmethod
    def verify(path):
        """Check the mcux installation path if it is valid!"""

        executable = get_executable(path)
        return os.path.exists(executable)







    @staticmethod
    def parse_build_result(exitcode, logfile=None):
        """Parse mcuxpressoide build result

        Arguments:
            exitcode {int} -- the exit code of mcuxpressoide process
            logfile {string} -- a file contains the output from mcuxpressoide.

        Returns:
            <BuildResult object> -- the build result

        Notes:
        ---------------------
        mcuxpressoide exit code
            noerror warnnings: 0
            erros: none-zero

        Parser the log file by search the keyword: "Total number of warnings: <>".
        """
        status = APP.EXITCODE.get(exitcode, "Errors")
        return BuildResult.map(status)






def get_executable(path):
    """Return mcuxpresso executable"""

    osname = platform.system()

    if osname == "Windows":
        return  os.path.join(path, "ide/mcuxpressoidec.exe").replace("\\", "/")

    elif osname == "Linux":
        return os.path.join(path , "ide/mcuxpressoide")

    elif osname == "Darwin":
        return os.path.join(path , "ide/MCUXpressoIDE.app/Contents/MacOS/mcuxpressoide")








def get_latest():
    """Find the latest installation of mcux in system.

    Windows:
        Installed at: in C:/nxp
    Linux:
        Installed at: /usr/local/
        Use Following command:
        >>> dpkg -L mcuxpressoide | grep /usr/local/mcuxpressoide | head -1
        >>> /usr/local/mcuxpressoide-1.0.0.277

    MAC:
        Installed at: /Applications
    """
    osname = platform.system()
    path, version = None, None

    if osname == "Windows":
        parent_dir = APP.default_install_directory()
        installations_list = [dir for dir in os.listdir(parent_dir) if "MCUXpressoIDE" in dir]
        if not installations_list:
            return None, None

        installations_list.sort()
        foldername = installations_list[-1]
        version = foldername.replace("MCUXpressoIDE_", "")
        path = parent_dir+foldername
        return version, path


    #==============================================================
    elif osname == "Linux":
        dirlist = os.listdir("/usr/local/")
        verlist = [edir for edir in dirlist if "mcuxpressoide" in edir]
        if not verlist:
            return None, None
        maxversion = max(verlist)
        version = maxversion.replace("mcuxpressoide-", "")
        path = "/usr/local/" + maxversion
        return version, path


    #==============================================================
    elif osname == "Darwin":
        dirlist = os.listdir("/Applications")
        mcux_pool = [dirname for dirname in dirlist if "MCUXpressoIDE" in dirname]
        if not mcux_pool:
            return None, None

        mcux_pool.sort()
        latest_installation_dirname = mcux_pool[-1]
        path = "/Applications/%s"%latest_installation_dirname
        version = latest_installation_dirname.replace("MCUXpressoIDE_", "")
        return version, path
